/*
    Copyright (C) 2010  Thomas Schrøder

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    LICENSE.txt file for license details.
*/

//////////////////////////////////////////////////////////////////////////
///// Different versions of CalcF corresponding to different potentials //
//////////////////////////////////////////////////////////////////////////

void Sample::CalcF(Potential *P, bool calc_stresses, bool calc_temp, bool calc_molp) {
  assert(allocatedNumParticles > 0);
  
  Pot_LJ_12_6* pLJ_12_6 = dynamic_cast<Pot_LJ_12_6 *>(P);
  Pot_LJ_12_6_Gromacs* pLJ_12_6_Gromacs = dynamic_cast<Pot_LJ_12_6_Gromacs*>(P);
  Pot_IPL_12* pIPL_12 = dynamic_cast<Pot_IPL_12*>(P);
  Pot_IPL_18* pIPL_18 = dynamic_cast<Pot_IPL_18*>(P);
  Pot_IPL_n* pIPL_n = dynamic_cast<Pot_IPL_n*>(P);
  Pot_gLJ_m_n* pgLJ_m_n = dynamic_cast<Pot_gLJ_m_n*>(P);
  Pot_LJ_12_6_NEW* pLJ_12_6_NEW = dynamic_cast<Pot_LJ_12_6_NEW*>(P);
  Pot_Gauss* pGauss = dynamic_cast<Pot_Gauss*>(P);
  Pot_LJ_12_6_Gauss* pLJ_12_6_Gauss = dynamic_cast<Pot_LJ_12_6_Gauss*>(P);
  Pot_Buckingham3* pBuckingham3 = dynamic_cast<Pot_Buckingham3*>(P);

  float4* d_r = particleData.d_r;
  float4* d_f = particleData.d_f;
  float4* d_w = particleData.d_w;
  float4* d_sts = particleData.d_sts;
  NBTYPEv *NbList = neighborList.NbList;

  if(pLJ_12_6) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pLJ_12_6, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pLJ_12_6, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pLJ_12_6_Gromacs) {
    if(calc_stresses)
      md_calc_f<1,1><<< grid, threads, shared_size >>>( pLJ_12_6_Gromacs, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,1><<< grid, threads, shared_size >>>( pLJ_12_6_Gromacs, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pIPL_12) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pIPL_12, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pIPL_12, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pIPL_18) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pIPL_18, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pIPL_18, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pIPL_n) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pIPL_n, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pIPL_n, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pgLJ_m_n) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pgLJ_m_n, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pgLJ_m_n, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pLJ_12_6_NEW) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pLJ_12_6_NEW, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pLJ_12_6_NEW, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pGauss) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pGauss, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pGauss, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pLJ_12_6_Gauss) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pLJ_12_6_Gauss, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pLJ_12_6_Gauss, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else if(pBuckingham3) {
    if(calc_stresses)
      md_calc_f<1,0><<< grid, threads, shared_size >>>( pBuckingham3, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
    else
      md_calc_f<0,0><<< grid, threads, shared_size >>>( pBuckingham3, d_r, d_f, d_w, d_sts, d_misc, d_L, d_invL, NbList );
  }
  else 
    {
      std::cerr << "Sample::CalcF : Failed dynamic_cast of potential pointer" << std::endl;
      exit(1);
    }
}
